@@ -49,7 +49,6 @@
 
 #include "hyperoid.h"
 
-
 static int palrgb[16 * 3] = {
     0, 0, 0, 128, 128, 128,
     192, 192, 192, 255, 255, 255,
@@ -1403,11 +1402,30 @@ void stop_sound(void)
 #endif
 }
 
-static gboolean
-game_tick_cb(gpointer data)
+void
+game_iteration(void)
 {
 CheckScore();
 DrawObjects();
+}
+
+static gboolean
+game_tick_cb(gpointer data)
+{
+struct timespec ts1;
+struct timespec ts2;
+gint t=50;
+glong td;
+
+clock_gettime(CLOCK_REALTIME, &ts1);
+game_iteration();
+
+#if 0
+while(gtk_events_pending())
+	gtk_main_iteration();
+#else
+gdk_flush();
+#endif
 
 if (bRestart) {
 	restart_timer_count--;
@@ -1416,23 +1434,20 @@ if (bRestart) {
 		RestartHyperoid();
 	}
 }
+clock_gettime(CLOCK_REALTIME, &ts2);
 
-if (IsKeyDown(KEY_F1))
-	NewGame();
+td=(ts2.tv_nsec<ts1.tv_nsec) ? 1000000000+ts2.tv_nsec-ts1.tv_nsec : ts2.tv_nsec-ts1.tv_nsec;
 
-if (IsKeyDown(KEY_ESC)) {
-	game_pause_tick();
-	gtk_main_quit();
-}
-
-return TRUE;
+t=CLAMP(t-td/1000000,10,50);
+tick_sid=g_timeout_add_full(G_PRIORITY_HIGH_IDLE, t, game_tick_cb, NULL, NULL);
+return FALSE;
 }
 
 void
 game_start_tick(void)
 {
 if (tick_sid==0)
-	tick_sid=g_timeout_add(50, game_tick_cb, NULL);
+	tick_sid=g_timeout_add_full(G_PRIORITY_HIGH_IDLE, 50, game_tick_cb, NULL, NULL);
 }
 
 void
